//
// Epilog - Generic text file analysis and forwarding
// Copyright 2001-2005 InterSect Alliance Pty Ltd
// http://www.intersectalliance.com/
//
#define _CRT_SECURE_NO_DEPRECATE 1
#define _CRT_NONSTDC_NO_DEPRECATE 1

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <winsock2.h>
#include <lm.h>

#include <comdef.h>
#include <activeds.h>

//#include <atlbase.h>
#include <windows.h>

#include "WebPages.h"
#include "Version.h"

#include "support.h"
#include "MD5.h"

#include "webserver.h"

extern void DebugMsg(const char* pszFormat, ...);

extern int SNAREDEBUG;
extern HANDLE hMutex;
extern EventCache *MCHead;

#define snprintf _snprintf

char WEB_CONFIG_FILENAME[MAX_AUDIT_CONFIG_LINE] = "\0";

// Make sure we return the size, or zero (for strings).
// Note that for the most part, the socket will be ignored
int HandleWebPages(char *HTTPBuffer,char *HTTPOutputBuffer,int size,SOCKET http_socket, HANDLE event)
{
	int returncode=0, refreshflag=0;
	char *ArgPosition, *pos, *pos2, *pos3;
	char TBuffer[2048]="";

	if(!HTTPBuffer || !HTTPOutputBuffer) {
		return(0);
	}

	// Stuff without the header/footer
	if(!strcmp(HTTPBuffer,"/intersect.gif")) {
		return(InterSectImage(HTTPBuffer,HTTPOutputBuffer,size));
	} else {
		char * pBuffer=HTTPOutputBuffer;
		int length=0;
		int psize=0;

		ArgPosition=strstr(HTTPBuffer,"?");
		// No arguments passed?
		if(!ArgPosition) {
			// Set argument position to the beginning of the buffer.
			ArgPosition=HTTPBuffer;
		} else {
			ArgPosition++;
		}

		if(!strncmp(HTTPBuffer,"/eventlog",9)) refreshflag=1;
		returncode=DefaultHeader(HTTPBuffer,pBuffer,size, refreshflag);
		length=(int)strlen(HTTPOutputBuffer);
		
		pBuffer+=length;
		psize=size-length;
		if(psize < 0) {
			psize=0;
		}

		// Check for a config argument, but only if you are the master
		pos2 = HTTPBuffer + 1;
		pos = strstr(pos2,"/");
		pos3 = strstr(pos2,"?");
		if (pos && (!pos3 || pos < pos3)) {
			HTTPBuffer = pos;
			strncpy(WEB_CONFIG_FILENAME, pos2, MAX_AUDIT_CONFIG_LINE);
			pos = strstr(WEB_CONFIG_FILENAME,"/");
			*pos = '\0';
		} else {
			WEB_CONFIG_FILENAME[0] = '\0';
		}

		// Web pages
		if(!strcmp(HTTPBuffer,"/network")) {
			returncode+=Network_Config(ArgPosition,pBuffer,psize);
		} else if(!strncmp(HTTPBuffer,"/network?",9)) {
			strncpy(pBuffer,"Nothing here yet",psize);
		} else if(!strcmp(HTTPBuffer,"/remote")) {
			returncode+=Remote_Config(ArgPosition,pBuffer,psize);
		} else if(!strncmp(HTTPBuffer,"/setremote",10)) {
			returncode+=Remote_Set(ArgPosition,pBuffer,psize);
		} else if(!strncmp(HTTPBuffer,"/objective",10)) {
			returncode+=Objective_Config(ArgPosition,pBuffer,psize);
		} else if(!strncmp(HTTPBuffer,"/setobjective",13)) {
			returncode+=Objective_Display(HTTPBuffer,pBuffer,psize);
		} else if(!strncmp(HTTPBuffer,"/changeobjective",16)) {
			returncode+=Objective_Result(HTTPBuffer,pBuffer,psize);
		} else if (!strncmp(HTTPBuffer, "/log", 4)) {
			returncode += Log_Config(ArgPosition, pBuffer, psize);
		} else if (!strncmp(HTTPBuffer, "/setlog", 7)) {
			returncode += Log_Display(HTTPBuffer, pBuffer, psize);
		} else if (!strncmp(HTTPBuffer, "/changelog", 10)) {
			returncode += Log_Result(HTTPBuffer, pBuffer, psize);
		} else if(!strcmp(HTTPBuffer,"/restart")) {
			returncode+=Restart(HTTPBuffer,pBuffer,psize,event);
		} else if(!strncmp(HTTPBuffer,"/restart?",10)) {
			strncpy(pBuffer,"Nothing here yet",psize);
		} else if(!strncmp(HTTPBuffer,"/setnetwork",11)) {
			returncode+=Network_Set(HTTPBuffer,pBuffer,psize);
		} else if(!strncmp(HTTPBuffer,"/status",6)) {
			returncode+=Status_Page(ArgPosition,pBuffer,psize);
		} else if(!strncmp(HTTPBuffer,"/eventlog",9)) {
			returncode+=Current_Events(ArgPosition,pBuffer,psize);
		} else if(strlen(TBuffer)) {
			strncpy(pBuffer,TBuffer,psize);
		} else if(!strcmp(HTTPBuffer,"/")) {
			returncode+=Status_Page(ArgPosition,pBuffer,psize);
		} else {
			// Here: Consider returning a 404 instead.
			// returncode=+Status_Page(ArgPosition,pBuffer,psize);
			if(SNAREDEBUG) { DebugMsg("No page - sending a 404"); }
			Display404(http_socket);
			return(-1);
		}

		pBuffer = HTTPOutputBuffer;
		length=(int)strlen(HTTPOutputBuffer);
		pBuffer+=length;
		psize=size-length;
		if(psize < 0) {
			psize=0;
		}

		returncode+=DefaultFooter(HTTPBuffer,pBuffer,psize);
	}

	return(returncode);
}


// This page will be usefull for debug information
int Status_Page(char *source, char *dest, int size)
{
	snprintf(dest,size,"<HTML><BODY><H2><CENTER>Epilog Version %s Status Page</H2></CENTER><P><center><font color=green>Epilog for Windows is currently active.</font></CENTER></BODY></HTML>",EPILOG_VERSION);
	return(0);
}

int Network_Config(char *source, char *dest, int size)
{
	//All strncpy or strncat functions in this routine have been designed avoid overflows
	Reg_Config config_struct;
	Reg_Network network_struct;
	DWORD dw_config_error,dw_network_error;
	char str_DestPort[10];
	char str_conferr[10],str_neterr[10];
	char *str_facility[] = {"Kernel","User","Mail","Daemon","Auth","Syslog","Lpr","News","UUCP","Cron","Authpriv","Ftp","Local0","Local1","Local2","Local3","Local4","Local5","Local6","Local7"};
	char *str_priority[] = {"Emergency","Alert","Critical","Error","Warning","Notice","Information","Debug"};
	UINT i,i_SyslogFacility,i_SyslogPriority;
		
	dw_config_error = Read_Config_Registry(&config_struct);
	dw_network_error = Read_Network_Registry(&network_struct);

	// This function will display the form used to set the audit configuration
	// The result of the form will be sent to "network_set"
	strncpy(dest,"<form method=get action=setnetwork><h2><center>SNARE Network Configuration</h2>",size);

	// Will display an error if unable to completely read from the registry
	if ((dw_network_error > 0) || (dw_config_error > 0)) {
		dw_network_error += WEB_READ_NETWORK_ERROR_CODE;
		dw_config_error += WEB_READ_CONFIG_ERROR_CODE;
		itoa(dw_network_error,str_neterr,10);
		itoa(dw_config_error,str_conferr,10);

		strncat(dest,"<br><b>NOTE: Some errors were encountered in reading the registry. Default values " \
					"may be used.<br> Report error: ",size - strlen(dest));
		strncat(dest,str_neterr,size - strlen(dest));
		strncat(dest,".",size - strlen(dest));
		strncat(dest,str_conferr,size - strlen(dest));
		strncat(dest,"</b><br>",size - strlen(dest));
	}

	strncat(dest,"<br>The following network configuration parameters of the SNARE unit is set to the following values:<br><br>" \
				"<table  width=70% border=0>" \
				"<tr bgcolor=#FFFFCC><td>Override detected DNS Name with: </td>" \
				"<td><input type=text name=str_ClientName size=25 value=\"",size - strlen(dest));
	
	strncat(dest,config_struct.str_ClientName,size - strlen(dest));
	strncat(dest,"\"></td></tr>",size - strlen(dest));


	// Here: Two alternatives: Allow a user-supplied comma-separated list, or
	// alternatively, add a new element for each new system?
	strncat(dest,"<tr bgcolor=#FFFFBB><td>Destination Snare Server address </td><td><input type=text name=str_Destination size=25 value=\"",size - strlen(dest));
	strncat(dest,network_struct.str_Destination,size - strlen(dest));
	strncat(dest,"\"></td></tr>",size - strlen(dest));

	strncat(dest,"<tr bgcolor=#FFFFCC><td>Destination Port <br>(if SYSLOG Header NOT enabled)</td><td><input type=text name=dw_DestPort size=8 value=\"",size - strlen(dest));
	itoa(network_struct.dw_DestPort,str_DestPort,10);
	strncat(dest,str_DestPort,size - strlen(dest));
	strncat(dest,"\"></td></tr>",size - strlen(dest));

	strncat(dest,"<tr bgcolor=#FFFFFF><td><br></td></tr>",size - strlen(dest));

	// First 3 bits = facility (eg: emerg, alert, etc.)
	// Rest = priority. Note that there is a gap between ftp and local1.
	i_SyslogPriority = (UINT)network_struct.dw_SyslogDest & 7;
	i_SyslogFacility = (UINT)network_struct.dw_SyslogDest >> 3;
	if(i_SyslogFacility > 11) { 
		i_SyslogFacility = i_SyslogFacility - 4;
	}

	//Need to convert this next section to YES or NO
	strncat(dest,"<tr bgcolor=#FFFFBB><td>Enable SYSLOG Header?</td><td><input type=checkbox name=dw_Syslog",size - strlen(dest));
	if (network_struct.dw_Syslog != 0) {
		strncat(dest," checked",size - strlen(dest));
	}
	strncat(dest,"></td></tr>",size - strlen(dest));

	strncat(dest,"<tr bgcolor=#FFFFCC><td>SYSLOG Facility </td><td><select name=SyslogFacility>",size - strlen(dest));
	for (i = 0; i < 19; i++)
	{
		strncat(dest,"<option",size - strlen(dest));
		if (i == i_SyslogFacility) {
			strncat(dest," selected>",size - strlen(dest));
		} else {
			strncat(dest,">",size - strlen(dest));
		}
		strncat(dest,str_facility[i],size - strlen(dest));
	}
	strncat(dest,"</select></td></tr>",size - strlen(dest));

	strncat(dest,"<tr bgcolor=#FFFFBB><td>SYSLOG Priority </td><td><select name=SyslogPriority>",size - strlen(dest));
	for (i = 0; i <= 7; i++) {
		strncat(dest,"<option",size - strlen(dest));
		if (i == i_SyslogPriority) {
			strncat(dest," selected>",size - strlen(dest));
		} else {
			strncat(dest,">",size - strlen(dest));
		}
		strncat(dest,str_priority[i],size - strlen(dest));
	}
	
	strncat(dest,"</select></td></tr>" \
		         "</table><br>" \
				 "<input type=submit value=\"Change Configuration\">    " \
				 "<input type=reset value=\"Reset Form\"></form>",size - strlen(dest));
	
	return(0);
}

int Network_Set(char *source, char *dest, int size)
{
	//All strncpy or strncat functions in this routine have been designed avoid overflows
	char *psource=source;
	char Variable[100]="", web_port[100]="",syslog_fac[100]="",syslog_pri[100]="";
	char Argument[100]="";
	char *str_facility[] = {"Kernel","User","Mail","Daemon","Auth","Syslog","Lpr","News","UUCP","Cron","Authpriv","Ftp","Local0","Local1","Local2","Local3","Local4","Local5","Local6","Local7"};
	char *str_priority[] = {"Emergency","Alert","Critical","Error","Warning","Notice","Information","Debug"};
	Reg_Network network_struct;
	Reg_Config config_struct;
	DWORD dw_error_network = 1,dw_error_config = 1,dw_SyslogClass = 0;
	int i;

	if(!source || !dest || !size) {
		return(0);
	}

	// This function will display that the remote audit configurations have been changed, or there have been errors
	strncpy(dest,"<h2><center>SNARE Network Configuration</h2>",size);

	// Note that all the possible variables do NOT have to be in the URL. The ones that are selected
	// via a checkbox will not be listed if the checkbox has been deselected.
	// Checking is limited to ensuring the Detsination port in the range 1-65535.
	// The variable associated with the checkbox (dw_Syslog) must be
	// exactly "on" or it will be defaulted to "off".

	network_struct.dw_DestPort = -1;
	network_struct.dw_SocketType=SOCKETTYPE_UDP;	// UDP
	network_struct.dw_Syslog = 0;

	while((psource=GetNextArgument(psource,Variable,sizeof(Variable),Argument,sizeof(Argument))) != (char *)NULL) 
	{	
		if (strstr(Variable,"str_ClientName") != NULL) {
			strncpy(config_struct.str_ClientName,Argument,sizeof(config_struct.str_ClientName));
		}
		if (strstr(Variable,"str_Destination") != NULL) {
			strncpy(network_struct.str_Destination,Argument,sizeof(network_struct.str_Destination));
		}
		if (strstr(Variable,"dw_DestPort") != NULL)	{
			strncpy(web_port,Argument,sizeof(web_port));
		}

		if (strstr(Variable,"SyslogPriority") != NULL) {
			strncpy(syslog_pri,Argument,sizeof(syslog_pri));
		}
		if (strstr(Variable,"SyslogFacility") != NULL) {
			strncpy(syslog_fac,Argument,sizeof(syslog_fac));
		}
		if (strstr(Variable,"dw_Syslog") != NULL) {
			if (strcmp(Argument,"on") == 0)
				network_struct.dw_Syslog = 1;
		}
	}

	for (i = 0; i <= 7; i++) {
		if (strstr(syslog_pri,str_priority[i]) != NULL) {
			network_struct.dw_SyslogDest = i;
			break;
		}
	}
	for (i = 0; i <= 19; i++) {
		if (strstr(syslog_fac,str_facility[i]) != NULL) {
			dw_SyslogClass = i;
			break;
		}
	}

	if(dw_SyslogClass > 11) {
		dw_SyslogClass = dw_SyslogClass + 4;
	}
	network_struct.dw_SyslogDest = network_struct.dw_SyslogDest | (dw_SyslogClass << 3);


	network_struct.dw_DestPort = atoi(web_port);
	if ((network_struct.dw_DestPort < 1) || (network_struct.dw_DestPort > 65535)) {
		strncat(dest,"The Destination Port value must be between 1 and 65535. Use the 'back' button to change the value.",size - strlen(dest));
	} else {

		dw_error_network = Write_Network_Registry(&network_struct);

		dw_error_config = Write_Config_Registry(&config_struct);

		if ((dw_error_network != 0) || (dw_error_config != 0)) {
			strncat(dest,"Values have NOT been changed.",size - strlen(dest));
		} else {
			strncat(dest,"Values have been changed.",size - strlen(dest));
		}
	}
	return(0);

}


int Remote_Config(char *source, char *dest, int size)
{
	//All strncpy or strncat functions in this routine have been designed avoid overflows
	Reg_Remote remote_struct;
	DWORD dw_remote_error;
	char str_WebPort[10];
	char str_remerr[10];

	dw_remote_error = Read_Remote_Registry(&remote_struct);

	// This function will display the form used to set the remote audit configuration
	strncpy(dest,"\n<form method=get action=setremote><h2><center>SNARE Remote Control Configuration</h2>",size);

	// Will display an error if unable to completely read from the registry
	if (dw_remote_error > 0)
	{
		dw_remote_error += WEB_READ_REMOTE_ERROR_CODE;
		itoa(dw_remote_error,str_remerr,10);
		
		strncat(dest,"<br><b>NOTE: Some errors were encountered in reading the registry. Default values " \
					"may be used.<br> Report error: ",size - strlen(dest));
		strncat(dest,str_remerr,size - strlen(dest));
		strncat(dest,"</b><br>\n",size - strlen(dest));
	}
	strncat(dest,"<br>The following remote control configuration parameters of the SNARE unit is set to the following values:<br><br>" \
		"<table  width=70% border=0>" \
		"<tr bgcolor=#FFFFCC><td>Allow remote control of SNARE agent</td><td><input type=checkbox name=dw_Allow",size - strlen(dest));

	if (remote_struct.dw_Allow != 0)
		strncat(dest," checked",size - strlen(dest));
	
	strncat(dest,"></td></tr>\n<tr bgcolor=#FFFFFF><td><br></td></tr>\n",size - strlen(dest));
	strncat(dest,"<tr bgcolor=#FFFFBB><td>Restrict remote control of SNARE agent to certain hosts </td><td><input type=checkbox name=dw_Restrict",size - strlen(dest));

	if (remote_struct.dw_Restrict != 0)
		strncat(dest," checked",size - strlen(dest));
	strncat(dest,"></td></tr>\n",size - strlen(dest));

	strncat(dest,"<tr bgcolor=#FFFFCC><td>IP Address allowed to remote control SNARE </td><td><input type=text name=str_RestrictIP size=12 value=\"",size - strlen(dest));
	strncat(dest,remote_struct.str_RestrictIP,size - strlen(dest));
	strncat(dest,"\"></td></tr>\n",size - strlen(dest));
	

	strncat(dest,"<tr bgcolor=#FFFFBB><td>Require a password for remote control? </td><td><input type=checkbox name=dw_Password",size - strlen(dest));

	if (remote_struct.dw_Password != 0)
		strncat(dest," checked",size - strlen(dest));
	strncat(dest,"></td></tr>\n",size - strlen(dest));

	strncat(dest,"<tr bgcolor=#FFFFCC><td>Password to allow remote control of SNARE </td><td><input type=password name=str_Password size=12 value=\"",size - strlen(dest));
	strncat(dest,remote_struct.str_Password,size - strlen(dest));
	strncat(dest,"\"></td></tr>\n",size - strlen(dest));


	strncat(dest,"<tr bgcolor=#FFFFBB><td>Change Web Server default (6162) port </td><td><input type=checkbox name=dw_PortChange",size - strlen(dest));

	if (remote_struct.dw_WebPortChange != 0)
		strncat(dest," checked",size - strlen(dest));
	strncat(dest,"></td></tr>\n",size - strlen(dest));

	strncat(dest,"<tr bgcolor=#FFFFCC><td>Web Server Port </td><td><input type=text name=dw_WebPort size=8 value=\"",size - strlen(dest));
	itoa(remote_struct.dw_WebPort,str_WebPort,10);
	strncat(dest,str_WebPort,size - strlen(dest));
	strncat(dest,"\"></td></tr>\n",size - strlen(dest));

	strncat(dest,"</table><br>\n",size - strlen(dest));
	strncat(dest,"<input type=submit value=\"Change Configuration\">    ",size - strlen(dest));
	strncat(dest,"<input type=reset value=\"Reset Form\"></form>",size - strlen(dest));
	
	return(0);
}

int Remote_Set(char *source, char *dest, int size) 
{
	// All strncpy or strncat functions in this routine have been designed avoid overflows
	char *psource=source;
	char Variable[100], web_port[100];
	char Argument[100];
	Reg_Remote remote_struct;
	DWORD dw_error = 0;
	
	dw_error = Read_Remote_Registry(&remote_struct);

	// This function will display that the remote audit configurations have been changed, or there have been errors
	strncpy(dest,"<h2><center>SNARE Remote Control Configuration</h2>",size);

	// Note that all the possible variables do NOT have to be in the URL. The ones that are selected
	// via a checkbox will not be listed if the checkbox has been deselected.
	// Also be aware that there may not be any arguments for this objective. If a sysadmin does not want
	// remote control, then there will be no arguments.
	// Hence: Checking is limited to Webport in the range 1-65535 only if portchange is "on"
	// The three variable associated with the checkboxes (dw_Allow, dw_Restrict, and dw_PortChange) must be
	// exactly "on" or they will be defaulted to "off".
	// str_RestrictIP can be anything it wants to be, so long as it is within size bounds.

	// Configure the defaults.
	remote_struct.dw_Allow=0;
	remote_struct.dw_WebPort = -1;
	remote_struct.dw_Password = 0;
	remote_struct.dw_Restrict = 0;
	remote_struct.dw_WebPortChange = 0;

	while((psource=GetNextArgument(psource,Variable,sizeof(Variable),Argument,sizeof(Argument))) != (char *)NULL) 
	{	
		if (strstr(Variable,"dw_WebPort") != NULL) {
			strncpy(web_port,Argument,sizeof(web_port));
		}
		if (strstr(Variable,"str_RestrictIP") != NULL) {
			strncpy(remote_struct.str_RestrictIP,Argument,sizeof(remote_struct.str_RestrictIP));
		}
		if (strstr(Variable,"str_Password") != NULL) {
			if(strcmp(Argument,remote_struct.str_Password)!=0) {
				// Changed
				// mD5 things here.
				strncpy(remote_struct.str_Password,MD5String(Argument),sizeof(remote_struct.str_Password));
			}
		}
		if (strstr(Variable,"dw_Allow") != NULL)
		{
			if (strcmp(Argument,"on") == 0)
				remote_struct.dw_Allow = 1;
			else
				remote_struct.dw_Allow = 0;
		}
		if (strstr(Variable,"dw_Password") != NULL)	{
			if (strcmp(Argument,"on") == 0) {
				remote_struct.dw_Password = 1;
			} else {
				remote_struct.dw_Password = 0;
			}
		}
		if (strstr(Variable,"dw_Restrict") != NULL)
		{
			if (strcmp(Argument,"on") == 0) {
				remote_struct.dw_Restrict = 1;
			} else {
				remote_struct.dw_Restrict = 0;
			}
		}
		if (strstr(Variable,"dw_PortChange") != NULL)
		{
			if (strcmp(Argument,"on") == 0) {
				remote_struct.dw_WebPortChange = 1;
			} else {
				remote_struct.dw_WebPortChange = 0;
			}
		}
	}

	remote_struct.dw_WebPort = atoi(web_port);
	if ((remote_struct.dw_WebPort < 1) || (remote_struct.dw_WebPort > 65535))
	{
		strncat(dest,"The Web Port value must be between 1 and 65535. Use the 'back' button to change the value.",size - strlen(dest));
	} else {
		// Check remote_struct.str_Password against the existing password.
		
		dw_error = Write_Remote_Registry(&remote_struct);
		if (dw_error != 0)
		{
			strncat(dest,"Remote Control Values have NOT been changed. Report error: ",size - strlen(dest));
			//**********PUT AN ERROR CODE IN HERE
		} else {
			strncat(dest,"Remote Control Values have been changed.",size - strlen(dest));
		}
	}
	return(0);
}


int Objective_Config(char *source, char *dest, int size)
{
	//All strncpy or strncat functions in this routine have been designed avoid overflows
	Reg_Objective reg_objective;
	DWORD dw_objective_error = 0;
	int i_objective_count = 0;
	char str_obj_count[10];
	char str_match_metachar_remove[SIZE_OF_GENERALMATCH*2];


	strncpy(dest,"<form method=get action=setobjective><H2><CENTER>SNARE Filtering Objectives Configuration</H2>",size);
		
	dw_objective_error = Read_Objective_Registry(i_objective_count,&reg_objective);
	if (dw_objective_error == 0)
	{
		strncat(dest,"<br>The following filtering objectives of the SNARE unit are active:<br><br>" \
		"<table  width=100% border=1>",size - strlen(dest));

		strncat(dest,"<tr bgcolor=#FFFFBB><center><td width=10%><b>Action Required</b></td>" \
			        "<td width=10%><b>User Include/Exclude</b></td><td><b>Search Term</b></td></center></tr>",size - strlen(dest));

		while (dw_objective_error == 0)
		{
			itoa(i_objective_count,str_obj_count,10);		
			
			if ((i_objective_count%2) == 0)
				strncat(dest,"<tr bgcolor=#FFFFCC><td><input type=submit name=",size - strlen(dest));
			else
				strncat(dest,"<tr bgcolor=#FFFFBB><td><input type=submit name=",size - strlen(dest));

			strncat(dest,str_obj_count,size - strlen(dest));
			strncat(dest," value=Delete>     ",size - strlen(dest));

			strncat(dest,"<input type=submit name=",size - strlen(dest));
			strncat(dest,str_obj_count,size - strlen(dest));
			strncat(dest," value=Modify>",size - strlen(dest));
			strncat(dest,"</td><td>",size - strlen(dest));


			// Debracket the strings in here. For HTML display purposes, the HTML metacharacters
			// need to be replaced. This is done with the "debracket" routine.
			// Note that the new strings are allowed to be twice as long as the real strings

			debracket(reg_objective.str_match,str_match_metachar_remove,SIZE_OF_GENERALMATCH*2);

			if(reg_objective.dw_match_type!=0) {
				strncat(dest,"Exclude",size - strlen(dest));
			} else {
				strncat(dest,"Include",size - strlen(dest));
			}
			strncat(dest,"</td><td>",size - strlen(dest));
			
			if (strlen(reg_objective.str_match) == 0) {
				strncat(dest,"&nbsp",size - strlen(dest));
			} else {
				strncat(dest,str_match_metachar_remove,size - strlen(dest));
			}
		
			strncat(dest,"</td></tr>",size - strlen(dest));

			i_objective_count++;
			dw_objective_error = Read_Objective_Registry(i_objective_count,&reg_objective);
		}
		strncat(dest,"</table><br>",size - strlen(dest));
	} else {
		strncat(dest,"<br>There are no current filtering objectives active.<br><br>",size - strlen(dest));
	}

	strncat(dest,"Select this button to add a new objective.  ",size - strlen(dest));
	strncat(dest,"<input type=submit name=0",size - strlen(dest));
	strncat(dest," value=Add>",size - strlen(dest));

	return(0);
}

int Objective_Display(char *source, char *dest, int size)
{
	//All strncpy or strncat functions in this routine have been designed avoid overflows
	Reg_Objective reg_objective;
	DWORD dw_objective_error = 0, dw_objective_write_error = 0,dw_objective_delete_error = 0;
;
	char str_objerr[10];
	int i_set=0,i_objective_count = 0,i_type = 0;
	char *psource=source, Variable[100], Argument[100];
	char str_temp[20], str_temp_objective[10];
	

	//This function will display an existing, or a blank, objective
	strncpy(dest,"<form method=get action=changeobjective><h2><center>SNARE Filtering Objective Configuration</h2>",size);

	//Determine whether the objective will be modified or deleted
	while((psource=GetNextArgument(psource,Variable,sizeof(Variable),Argument,sizeof(Argument))) != (char *)NULL) 
	{	
		if (strstr(Argument,"Delete") != NULL)
		{
			sscanf(Variable,"%20[^?]?%10[^\n]\n",str_temp,str_temp_objective);
			i_type = 0;
			break;
		}
		if (strstr(Argument,"Modify") != NULL)
		{
			sscanf(Variable,"%20[^?]?%10[^\n]\n",str_temp,str_temp_objective);
			i_type = 1;
			break;
		}
		if (strstr(Argument,"Add") != NULL)
		{
			strncpy(str_temp_objective,"-2",sizeof(str_temp_objective));
			i_type = 2;
			break;
		}
	}

	//Extract the objective number. I have to do this stuff, because atoi returns 0 if it cannot convert the string
	if (strcmp(str_temp_objective,"0") == 0)
		i_objective_count = -1;
	else
		i_objective_count = atoi(str_temp_objective);

	//If the objective number could not be successfully extracted, return immediately.
	if (i_objective_count == 0)
	{
		strncat(dest,"<br><b>NOTE: It appears the URL is encoded incorrectly.",size - strlen(dest));
		return 0;
	}

	if (i_objective_count == -1)
		i_objective_count = 0;
		
	//If the objective is being modified or added
	if (i_type > 0) {
		if (i_type == 1) {
			dw_objective_error = Read_Objective_Registry(i_objective_count,&reg_objective);
		} else {
			strncpy(reg_objective.str_match,"*",sizeof(reg_objective.str_match));
			strncpy(reg_objective.str_match_type,INCLUDE,sizeof(reg_objective.str_match_type));
		}

		// Will display an error if unable to completely read from the registry
		if (dw_objective_error > 0) {
			dw_objective_error += WEB_READ_OBJECTIVE_ERROR_CODE;
			itoa(dw_objective_error,str_objerr,10);
	
			strncat(dest,"<br><b>NOTE: Some errors were encountered in reading the registry. Default values " \
						"may be used.<br> Report error: ",size - strlen(dest));
			strncat(dest,str_objerr,size - strlen(dest));
			strncat(dest,"</b><br>",size - strlen(dest));
		}

		strncat(dest,"<br>The following parameters of the SNARE objective may be set:<br><br>" \
			"<table  width=100% border=0>",size - strlen(dest));

		//Identify the high level event. Note that there is a table within a table in these radio buttons.
		i_set = 0;

		strncat(dest,"<tr bgcolor=#FFFFBB><td>General Search Term<br><i>Wildcards accepted</i></td><td><input type=text name=str_match size=50 value=\"",size - strlen(dest));
		strncat(dest,reg_objective.str_match,size - strlen(dest));
		strncat(dest,"\"></td></tr>",size - strlen(dest));

		strncat(dest,"<tr bgcolor=#FFFFCC><td>Select the User Match Type</td><td>",size - strlen(dest));
		strncat(dest,"<input type=radio name=str_match_type value=",size - strlen(dest));
		strncat(dest,INCLUDE,size - strlen(dest));
		if (strstr(reg_objective.str_match_type,INCLUDE) != NULL)
			strncat(dest," checked",size - strlen(dest));
		strncat(dest,">Include    <input type=radio name=str_match_type value=",size - strlen(dest));
		strncat(dest,EXCLUDE,size - strlen(dest));
		if (strstr(reg_objective.str_match_type,EXCLUDE) != NULL)
			strncat(dest," checked",size - strlen(dest));
		strncat(dest,">Exclude    </td></tr>",size - strlen(dest));

		//Identify the event type to capture. Note that there is a table within a table in these radio buttons.
		i_set = 0;

		//Identify the log type to capture. Note that there is a table within a table in these radio buttons.
		i_set = 0;

		strncat(dest,"</table><br>",size - strlen(dest));
		strncat(dest,"<input type=hidden name=objnumber value=",size - strlen(dest));
		strncat(dest,str_temp_objective,size - strlen(dest));//Objective number goes here
		strncat(dest,"><input type=submit value=\"Change Configuration\">    ",size - strlen(dest));
		strncat(dest,"<input type=reset value=\"Reset Form\"></form>",size - strlen(dest));
	} else {
		dw_objective_delete_error = Delete_Objective(i_objective_count);
		i_objective_count++;
		dw_objective_error = Read_Objective_Registry(i_objective_count,&reg_objective);
		while (dw_objective_error == 0)
		{
			dw_objective_write_error = Write_Objective_Registry(i_objective_count-1,&reg_objective);
			dw_objective_delete_error = Delete_Objective(i_objective_count);
			i_objective_count++;
			dw_objective_error = Read_Objective_Registry(i_objective_count,&reg_objective);
		}

		if (dw_objective_delete_error == 0) {
			strncat(dest,"<br>The objective has been deleted.",size - strlen(dest));
		} else {
			strncat(dest,"<br>The objective was unable to be deleted.",size - strlen(dest));
			//***REPORT AN ERROR
		}
	}
	
	return(0);
}


int Objective_Result(char *source, char *dest, int size)
{
	//All strncpy or strncat functions in this routine have been designed avoid overflows
	Reg_Objective reg_objective, reg_objective_read;
	// DWORD dw_objective_error = -1;
	DWORD dw_objective_error = 0;

	int i_objective_count = 0,i_objective = 0, i_event_type_set = 0, i_event_type_log_set = 0;
	char str_obj_count[10];
	char *psource=source, Variable[100], Argument[100];

	strncpy(dest,"<form method=get action=setobjective><H2><CENTER>SNARE Filtering Objectives Configuration</H2>",size);

	while((psource=GetNextArgument(psource,Variable,sizeof(Variable),Argument,sizeof(Argument))) != (char *)NULL) 
	{	
		if (strstr(Variable,"str_match_type") != NULL) {
			strncpy(reg_objective.str_match_type,Argument,sizeof(reg_objective.str_match_type));
			continue;
		}
		if (strstr(Variable,"str_match") != NULL) {
			strncpy(reg_objective.str_match,Argument,sizeof(reg_objective.str_match));
		}
		if (strstr(Variable,"objnumber") != NULL) {
			strncpy(str_obj_count,Argument,sizeof(str_obj_count));
		}
	}

	if(!dw_objective_error) {
		i_objective = atoi(str_obj_count);
		
		//-2 = "Add a new objective", hence we must go to the end of the list.
		if (i_objective == -2)
		{
			dw_objective_error = Read_Objective_Registry(i_objective_count,&reg_objective_read);
			while (dw_objective_error == 0)
			{
				i_objective_count++;
				dw_objective_error = Read_Objective_Registry(i_objective_count,&reg_objective_read);
			}
			i_objective = i_objective_count;
		}

		dw_objective_error = Write_Objective_Registry(i_objective,&reg_objective);
		if (dw_objective_error  == 0)
			strncat(dest,"<br>The objective has been modifed/added.",size - strlen(dest));
		else
			strncat(dest,"<br>The objective was unable to be modifed/added.",size - strlen(dest));
			//***REPORT AN ERROR
	}

	return(0);
}


int DefaultHeader(char *source, char *dest, int size, int refreshflag)
{
	//All strncpy or strncat functions in this routine have been designed avoid overflows
	strncpy(dest,"<HTML><head>" \
	"<title>InterSect Alliance - Information Technology Security</title>" \
	"<meta name=\"TITLE\" content=\"InterSect Alliance - Information Technology Security\">" \
	"<style type=\"text/css\">\n" \
	"body {\n" \
	" font-family: Verdana,Helvetica,sans-serif;\n" \
	" font-size: 10px; font-weight: normal;\n" \
	" margin: 0px;\n" \
	"}\n" \
	"h1 {\n" \
	" font-size: 30px;\n" \
	" font-family: Verdana,Helvetica,sans-serif;\n" \
	" color: white;\n" \
	"}\n" \
	"h2 {\n" \
	" font-size: 30px;\n" \
	" font-family: Verdana,Helvetica,sans-serif;\n" \
	" color: black;\n" \
	"}\n" \
	"font {\n" \
	" font-family: Verdana,Helvetica,sans-serif;\n" \
	" text-decoration: none; font-size: 10px;\n" \
	" font-weight: normal;\n" \
	"}\n" \
	"table {\n" \
	" margin: 0px; padding: 0px;\n" \
	"}\n" \
	"td {\n" \
	" font-size: 75%;\n" \
	"}\n" \
	"</style>\n" \
	"<meta HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">" \
	"<meta HTTP-EQUIV=\"Expires\" CONTENT=\"-1\">" ,size);
	if (refreshflag) strncat(dest,"<meta HTTP-EQUIV=\"Refresh\" CONTENT=\"10\">",size);
	strncat(dest,"</head>" \
	"<body text=black bgcolor=white link=#000066 vlink=#000044 alink=#000055>" \
	"<table border=0 cellspacing=0 cellpadding=0 columns=3 width=100%>" \
	"<tbody>" \
    "<tr>" \
    "<td height=70><img src=/intersect.gif alt=\"InterSect\" width=205 height=70 hspace=20 vspace=0 border=0 align=Right>" \
    "</td>" \
    "<td height=70 width=1% bgcolor=#eeeeee><br></td>" \
	"<td height=70 width=1% bgcolor=#dddddd><br></td>" \
	"<td height=70 width=1% bgcolor=#cccccc><br></td>" \
	"<td height=70 width=1% bgcolor=#ccbbbb><br></td>" \
	"<td height=70 width=1% bgcolor=#ccaaaa><br></td>" \
	"<td height=70 width=1% bgcolor=#cc9999><br></td>" \
	"<td height=70 width=1% bgcolor=#cc8888><br></td>" \
	"<td height=70 width=1% bgcolor=#cc7777><br></td>" \
	"<td height=70 width=1% bgcolor=#cc6666><br></td>" \
	"<td height=70 width=1% bgcolor=#cc5555><br></td>" \
	"<td height=70 width=1% bgcolor=#cc4444><br></td>" \
	"<td height=70 width=1% bgcolor=#cc3333><br></td>" \
	"<td height=70 width=1% bgcolor=#cc2222><br></td>" \
	"<td height=70 width=1% bgcolor=#cc1111><br></td>" \
	"<td height=70 width=86% bgcolor=#cc0000><div align=right><center><h1>Epilog for Windows&nbsp;</h1></center></div></td>" \
    "</tr>" \
    "</tbody>" \
	"</table>" \
	"<table border=0 cellspacing=0 cellpadding=0 columns=1 width=100%>" \
    "<tr><td height=3 border=0 bgcolor=white width=100%></td></tr>" \
    "<tr><td height=3 border=0 bgcolor=black width=100%></td></tr>" \
    "<tr><td height=2 border=0 bgcolor=#AAAAAA width=100%></td></tr>" \
    "<tr><td height=2 border=0 bgcolor=white width=100%></td></tr>" \
	"</table>" \
	"<table border=0 cellspacing=0 cellpadding=5 columns=2 width=100% height=100%>" \
	"<tbody>" \
    "<tr>" \
      "<td valign=Top width=20% bgcolor=#cc0000>" \
      "<div align=Center><font color=#ffffff>" \
      "<br>" \
      "<font face=\"Helvetica,Arial,sans-serif\" size=-1><B>" \
      "<br><A HREF=\"/eventlog\" style=\"color:FFFFFF;text-decoration:none\">Latest Events</A><br>" \
      "<br><A HREF=\"log\" style=\"color:FFFFFF;text-decoration:none\">Log Configuration</A><br>\n" \
      "<br><A HREF=\"/network\" style=\"color:FFFFFF;text-decoration:none\">Network Configuration</A><br>" \
      "<br><A HREF=\"/remote\" style=\"color:FFFFFF;text-decoration:none\">Remote Control Configuration</A><br>" \
      "<br><A HREF=\"/objective\" style=\"color:FFFFFF;text-decoration:none\">Objectives Configuration</A><br>" \
      "<br><A HREF=\"/status\" style=\"color:FFFFFF;text-decoration:none\">View Audit Service Status</A><br>" \
	  "<br><A HREF=\"/restart\" style=\"color:FFFFFF;text-decoration:none\">Apply the Latest Audit Configuration</A><br>" \
	  "</div>" \
      "</b></font>" \
      "<br>" \
      
    "</td>" \
    "<td width=100% valign=Top>" \
     "<table cellpadding=0 cellspacing=10 border=0 width=100%>" \
      "<tbody>" \
       "<tr>" \
        "<td valign=Top align=Justify>",size);
//   This line is used for SnareApache and SnareSquid support.
//      "<br><A HREF=\"switch\" style=\"color:FFFFFF;text-decoration:none\">Switch Configuration Files</A><br>\n" \

	return(0);
}

int DefaultFooter(char *source, char *dest, int size)
{
	//All strncpy or strncat functions in this routine have been designed avoid overflows
	strncpy(dest,"</td>" \
		"</tr>" \
		"</tbody>" \
		"</table>" \
		"</body><center>" \
		"<BR><BR><FONT SIZE=-1 face=helvetica>(c) <A HREF=\"http://www.intersectalliance.com\">Intersect Alliance</A> Pty Ltd 1999-2005. " \
		"This site is powered by <A HREF=\"http://www.intersectalliance.com/projects/\">SNARE for Windows.</A></FONT>" \
		"</center></html>",size);

	return(0);
}

int Restart(char *source, char *dest, int size, HANDLE event)
{
	// All strncpy or strncat functions in this routine have been designed avoid overflows
	strncpy(dest,"<HTML><BODY><H2><CENTER>Reapply the Latest Configuration</H2><P>Snare Objectives have been reapplied to the running system.</CENTER></BODY></HTML>",size);

	// Notify the main thread that we want to reapply config changes
	SetEvent(event);

    return(0);
}

HRESULT VarToBytes(VARIANT Variant,LPBYTE *bytes,long *plcb)
{
    HRESULT hr = E_FAIL;
    SAFEARRAY *pArrayVal = NULL;
    CHAR HUGEP *pArray = NULL;

	if(!plcb) {
		return(hr);
	}

    // Retrieve the safe array....
    pArrayVal = Variant.parray;
	
    if (pArrayVal != NULL) {
		long cSize;

		if(!Variant.parray->rgsabound) {
			if(SNAREDEBUG) { DebugMsg("WARNING: Variant array is corrupted. I'm not touching it."); }
			return(E_FAIL);
		}

		// try / catch
		try {
			cSize = pArrayVal->rgsabound[0].cElements;
		} catch(...) {
			if(SNAREDEBUG) { DebugMsg("CRASH: cSize grab failed. Some null-pointer wierdness going on here."); }
			return(E_FAIL);
		}

		// Just a small 'what the?' check
		if(cSize > 16384 || cSize <= 0) {
			if(SNAREDEBUG) { DebugMsg("Size: %d is just plain silly. Must be a corruption. Exiting",cSize); }
			return(E_FAIL);
		}

        *bytes = (LPBYTE)malloc( cSize );

        if( *bytes == NULL ) return E_FAIL;

        hr = SafeArrayAccessData(pArrayVal, (void HUGEP * FAR *) &pArray);
        if (SUCCEEDED(hr)) {
            // Copy the bytes to the safe array.
		    memcpy( *bytes, pArray, cSize );
		    SafeArrayUnaccessData( pArrayVal );
            *plcb = cSize;
            hr = S_OK;
        } else {
			// Clean up
			free(*bytes);
			return(E_FAIL);
		}
    } else {
        hr = E_OUTOFMEMORY;
    }

    return hr;
}


// Lets go about this VERY differently. It appears that getting a users group membership
// is relatively easy. Lets work backwards from that.
//
// General instructions:
// $ expr `./base64 intersect.gif | wc -c` - `./base64 intersect.gif | wc -l` "*" 2 + 1
//   2029
// ./base64 intersect.gif | sed 's/^/   "/' | sed 's/.$/"  \\/' >out.txt
// 
//  mangle out.txt as appropriate - add the (wc -c) - ((wc -l)*2) + 1 figure to the buffer size.
int InterSectImage(char *source, char *dest, int size)
{
	char IntersectGif[2029];
	char temp[2029];
	int size2;

	strncpy(IntersectGif,"R0lGODlhzQBGALMAAB8aF01JR2toZtwrGYF+fONVRo+Mi5eWleuCePKzrLOysszLy/jRzunl" \
	    "5f////j29SwAAAAAzQBGAEAI/gAdCBxIsKDBgw4eIFxIUCHDhxAjSkT4AMGAAQgmatw40CHH" \
	    "jxs9ghQJsuTEBxdTpkxgsqXLlzBjyvyoMuXMmzAB6NQp0MDOnzoDkOz504ADn0CTAniAVOlO" \
		"oQqcSgVqdKpVAARrXiyA0+XQkkCJ7jRakMDPAA3MjmUYtikAsgfV6lTw9WFYgwKACj34gIFW" \
		"jF0DCx5MeGPUsw08WtTKMqFcAGgZNghQ9ChQAQ1Kuq36s7DnzxL7/kXw4DFah6gFpk4o8ADV" \
		"Bwpjs45d1/Lau6Bz6y5YQCvXmbV3Cx++MUHvvxcZHFSgoOGB5g4WGFhAfUFmAwqqL2i94EGD" \
		"AwSZ/nNXreDA9gXmq5vvmb165sDBIcYnfjAB8gLz6ev3up+ifa0i5VeYgCEtROBIsyUIm0Gy" \
		"seZgbatFqNqEBB2nUkb9BXbXZgJN9hN0YukE10FtVYbQY1clxVmKSjW0WE0HZijRhiYetIBS" \
		"AdjVGYcUPZAXi0Bth9tBHv4EnkEWqjQUU5SxKMB2HTEFpJBXrTiVjFhO9F9NGKIIpE7g8ahZ" \
		"ZUNmaeZGL6rUkQJNfgllQmyyCKJBHJZ55p0LNZBkSr8VhN4D66mGnkGBEjRdRwcUetQB7zmw" \
		"HnraBYqddo3ieecDe2IUo6WcdtrRlkp6KuqoqmH6F34ZbkrqpZkOoOqq/7DqZmpNjcVqK55/" \
		"VSrjqyR2BmdSkSHEoVtS2TbVXkV+CQB0yho5EKgp6XorQmFJudZjBJAkZq87GfvWQo/NKWyN" \
		"8jnQwJDQXlTrtNT6KuYDN/6UmLcjGlTiWic2K68D+s5VEEq+Ycmrve6S21GbKrK1o8Edndsv" \
		"AN3ZqRrCAAiAUJopKcfuTQ0oYMChGwsU709vCuRXwAMl2+x7FF9FpVVWFhsyfMRaPBByDj0m" \
		"wMc898wzwgZYK2JLdfo6s4ytKqSzz0z3zGiIbw1cdLdH75rpbz/qC2XWci40NU9Vy4jxRWGX" \
		"/dnYA6zLWgPW8fUvTEtS5KDZoD2QLp/LpYceiP6DFqRoT1B+pxBzDk33HV2PHkloon/TLRhy" \
		"KzkuuWB9JYnh5C8N7OndgOmnOeaUN3DfgqCb9DmenDd6eulho315qpfSLRpjq7NOd6av244g" \
		"f2f+RXpH+9UON+h/6W78RpmqPXfIuMHbZAAEzMvXsCkKLdVesBnQslRPDtSv4g6MjWqWtd/l" \
		"MLDSQm0UsU5565RQD2zPYmYPL9XhXwl4JHx/NJ71k83A2xbBuiVAgpgmfSkDmmGGhLb9Yal/" \
		"Q3MAwsBHL4URkGEDWVrTeqbADW6QYvo7FfliAsFvdShIhcOg9xaGr7jUDzwPyxFB7pY/s5Ww" \
		"XocJynVUKJB7RZBBGv70oNPeQxWfcU1E3TlIAx24nxsy6IhUsSBPCphBxDyIQqvpodEKckQZ" \
		"HkSEsMtJwVpYkPMlpWQDnCIPHQDFKfFri2WkWL1MNpoeXfGOEIINbRS0PBKO8YfL0QsCV3hB" \
		"MiIkTsqKGBwNYkZ/FQRtDBCJyoAEovi56QEpiln7jheRHO4EjUkbiCebZZSRTQmTVXqjVTg5" \
		"EdecxSMnq0mfwgUR6j1FamRaJCs5ogAC9OxNaGuMl760vqLg8ja63CXcTqW0+u3QkBz5GlaU" \
		"2RVIluZDH3GLAAg0FGlSsysA45K56tedRqaILuNCJtW+eRPxqaaNKUqiJSlZF2+ys53FA6ne" \
		"PY3nzgLtM2TeqeM/jxdLGDHIY+Jim0GsQ5K2pQyNDhVZZuClnTdVlKED1UhsGNCqAWjMOdsp" \
		"j0ekQyc0OupNzBHcQJ5zKECdZ0QONQ8TOVkRyA1AV94xAOMSZYD3DEokMvVbEj2203mt5wHM" \
		"OVTfBJId7ugvoxJBm0qUZ66IdqhtbKNUdCoq0ehUCl6ZsWrbKKodh1x0kFAliFQL8NG0lm5L" \
		"CJAe3QICADs=",2029);

	size2=base64decode (temp, IntersectGif);

	memcpy(dest,temp,size2);
	return(size2);
}

int debracket(char *source, char *dest, int length)
{
	//This routine is simply to replace the HTML metacharacters "<" and ">" with
	// "&lt;" and "&gt;". That's all!

      int count=0;
      char *copyofdest;
      copyofdest=dest;

      if(!source || !dest) return(1);

      while(*source) {
              if(*source == '<') {
                      if(count < length) { *dest='&'; dest++; count++; }
                      if(count < length) { *dest='l'; dest++; count++; }
                      if(count < length) { *dest='t'; dest++; count++; }
                      if(count < length) { *dest=';'; dest++; count++; }
              } else if(*source == '>') {
                      if(count < length) { *dest='&'; dest++; count++; }
                      if(count < length) { *dest='g'; dest++; count++; }
                      if(count < length) { *dest='t'; dest++; count++; }
                      if(count < length) { *dest=';'; dest++; count++; }
              } else {
                      if(count < length) { *dest=*source; dest++; count++;}
              }
              source++;
      }
      *dest='\0';

	  dest=copyofdest;

      return(0);
}

int Display404(SOCKET http_socket)
{
	char HTTPBuffer[512];
	// Overwrite HTTPBuffer with the header data.
	strncpy(HTTPBuffer, "HTTP/1.0 404 Not Found\r\n" \
						"Server: SNARE/1.0\r\n" \
						"MIME-version: 1.0\r\n" \
						"Content-type: text/html\r\n\r\n" \
						"<html><body><center><h2>Page Not Found</h2></center></body></html>",
			sizeof(HTTPBuffer));

	return(send(http_socket,HTTPBuffer,(int)strlen(HTTPBuffer),0));
}

int DisplayTextHeader(SOCKET http_socket)
{
	char HTTPBuffer[512];
	// Overwrite HTTPBuffer with the header data.
	strncpy(HTTPBuffer, "HTTP/1.0 200 OK\r\n" \
						"Server: SNARE/1.0\r\n" \
						"MIME-version: 1.0\r\n" \
						"Content-type: text/plain\r\n\r\n",
			sizeof(HTTPBuffer));
	return(send(http_socket,HTTPBuffer,(int)strlen(HTTPBuffer),0));
}

int Log_Config(char *source, char *dest, int size)
{
	Reg_Log log_struct;
	int i_log_count= 0;
	DWORD dw_log_error;
	char str_log_count[10];
	char str_name_metachar_remove[MAX_AUDIT_CONFIG_LINE * 2];
	char str_format_metachar_remove[MAX_AUDIT_CONFIG_LINE * 2];

	FILE *configfile = (FILE *) NULL;

	if (!source || !dest || !size) {
		return(0);
	}

	strcpy(log_struct.name, "");
	strcpy(log_struct.type, "");

	strncpy(dest,
		"<form action=setlog><H2><CENTER>SNARE Log Configuration</H2>",
		size);

	dw_log_error = Read_Log_Registry(i_log_count,&log_struct);
	if (dw_log_error == 0)
	{
		strncat(dest,
			"<br>The following log files are being monitored by SNARE:<br><br>"
			"<table  width=100% border=1>", size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFBB><center><td width=\"10%\"><b>Action Required</b></td><td width=\"20%\"><b>Log Type</b></td>"
			"<td width=\"50%\"><b>Log File or Directory</b></td>"
			"<td width=\"20%\"><b>Log File Format</b></td>"
			"</center></tr>", size - strlen(dest));

		while (dw_log_error == 0) {
			snprintf(str_log_count, 10, "%d", i_log_count);

			if ((i_log_count) == 0)
				strncat(dest,
					"<tr bgcolor=#FFFFCC><td><input type=submit name=",
					size - strlen(dest));
			else
				strncat(dest,
					"<tr bgcolor=#FFFFBB><td><input type=submit name=",
					size - strlen(dest));

			strncat(dest, str_log_count, size - strlen(dest));
			strncat(dest, " value=Delete>     ",
				size - strlen(dest));

			strncat(dest, "<input type=submit name=",
				size - strlen(dest));
			strncat(dest, str_log_count, size - strlen(dest));
			strncat(dest, " value=Modify>", size - strlen(dest));
			strncat(dest, "</td><td>", size - strlen(dest));

			strncat(dest, log_struct.type,
				size - strlen(dest));
			strncat(dest, "</td><td>", size - strlen(dest));

			// Debracket the strings in here. For HTML display purposes, the HTML metacharacters
			// need to be replaced. This is done with the "debracket" routine.
			// Note that the new strings are allowed to be twice as long as the real strings
			debracket(log_struct.name,
				  str_name_metachar_remove,
				  MAX_AUDIT_CONFIG_LINE * 2);
			debracket(log_struct.format,
				  str_format_metachar_remove,
				  MAX_AUDIT_CONFIG_LINE * 2);

			if (strlen(log_struct.name) == 0) {
				strncat(dest, "&nbsp", size - strlen(dest));
			} else {
				strncat(dest, str_name_metachar_remove,
					size - strlen(dest));
			}
			strncat(dest, "</td><td>", size - strlen(dest));

			if (strlen(log_struct.format) == 0) {
				strncat(dest, "&nbsp", size - strlen(dest));
			} else {
				strncat(dest, str_format_metachar_remove,
					size - strlen(dest));
			}
			strncat(dest, "</td></tr>", size - strlen(dest));

			i_log_count++;
			dw_log_error = Read_Log_Registry(i_log_count,&log_struct);
		}
		strncat(dest, "</table><br>", size - strlen(dest));
	} else {
		strncat(dest,
			"<br>There are no current log monitors active.<br><br>",
			size - strlen(dest));
	}

	strncat(dest, "Select this button to add a new log monitor.  ",
		size - strlen(dest));
	strncat(dest, "<input type=submit name=0", size - strlen(dest));
	strncat(dest, " value=Add>", size - strlen(dest));

	return (0);
}

int Log_Display(char *source, char *dest, int size)
{
	Reg_Log log_struct;
	int dw_log_error = 0, dw_log_delete_error = 0, dw_log_write_error = 0;
	char str_logerr[10];
	int i_log_count = 0, i_type = 0, i;
	char *psource = source, Variable[100], Argument[100];
	char str_temp[20], str_temp_log[10];
	int log_type_count = 9;		// Don't forget to change this when adding new types
	char *str_value[] = {"GenericLog", "ApacheLog", "ExchMTLog", "IISWebLog", "ISAFwsLog", "ISAWebLog", "MSProxySvr", "SMTPSvcLog", "SquidProxyLog"};
	char *str_descr[] = {"Generic log format (default)", "Apache web logs", "Exchange message tracking logs", "Microsoft IIS web server logs", "Microsoft ISA firewall logs", "Microsoft ISA web logs", "Microsoft proxy server logs", "Microsoft SMTP logs", "Squid proxy logs"};

	// This function will display an existing, or a blank, log
	strncpy(dest,
		"<form action=changelog><h2><center>SNARE Log Configuration</h2>",
		size);

	// Determine whether the log will be modified or deleted
	while ((psource =
		GetNextArgument(psource, Variable, sizeof (Variable), Argument,
				sizeof (Argument))) != (char *) NULL) {
		if (strstr(Argument, "Delete") != NULL) {
			sscanf(Variable, "%20[^?]?%10[^\n]\n", str_temp,
			       str_temp_log);
			i_type = 0;
			break;
		}
		if (strstr(Argument, "Modify") != NULL) {
			sscanf(Variable, "%20[^?]?%10[^\n]\n", str_temp,
			       str_temp_log);
			i_type = 1;
			break;
		}
		if (strstr(Argument, "Add") != NULL) {
			strncpy(str_temp_log, "-2",
				sizeof (str_temp_log));
			i_type = 2;
			break;
		}
	}

	// Extract the log number. I have to do this stuff, because atoi returns 0 if it cannot convert the string
	if (strcmp(str_temp_log, "0") == 0)
		i_log_count = -1;
	else
		i_log_count = atoi(str_temp_log);

	// If the log number could not be successfully extracted, return immediately.
	if (i_log_count == 0) {
		strncat(dest,
			"<br><b>NOTE: It appears the URL is encoded incorrectly.",
			size - strlen(dest));
		return 0;
	}

	if (i_log_count == -1)
		i_log_count = 0;

	// If the log is being modified or added
	if (i_type > 0) {
		if (i_type == 1) {
			dw_log_error = Read_Log_Registry(i_log_count,&log_struct);
		} else {
			// Defaults
			strncpy(log_struct.name, "", sizeof(log_struct.name));
			strncpy(log_struct.type, "GenericLog", sizeof(log_struct.type));
			strncpy(log_struct.format, "", sizeof(log_struct.format));
		}

		// Will display an error if unable to completely read from the config file
		if (dw_log_error > 0) {
			dw_log_error += WEB_READ_LOG_ERROR_CODE;
			snprintf(str_logerr, 10, "%d", dw_log_error);

			strncat(dest,
				"<br><b>NOTE: Some errors were encountered in reading the configuration file. Default values "
				"may be used.<br> Report error: ",
				size - strlen(dest));
			strncat(dest, str_logerr, size - strlen(dest));
			strncat(dest, "</b><br>", size - strlen(dest));
		}

		strncat(dest,
			"<br>The following parameters of the SNARE log inputs may be set:<br><br>"
			"<table  width=100% border=0>", size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFCC><td>Select the Log Type</td><td><select name=str_log_type>",
			size - strlen(dest));

		for (i = 0; i < log_type_count; i++) {
			strncat(dest, "<option value=", size - strlen(dest));
			strncat(dest, str_value[i], size - strlen(dest));
			if (strncmp(str_value[i], log_struct.type, MAX_AUDIT_CONFIG_LINE) == 0)
				strncat(dest, " selected>", size - strlen(dest));
			else
				strncat(dest, ">", size - strlen(dest));
			strncat(dest, str_descr[i], size - strlen(dest));
		}

		/*if (strcmp(log_struct.str_general_match, ".*") == 0) {
			strncat(dest, " checked", size - strlen(dest));
			strncpy(log_struct.str_general_match_type, "Any", sizeof(log_struct.str_general_match_type));
		}
		strncat(dest, ">Match Any String    ", size - strlen(dest));
		*/
		strncat(dest, "</select></td></tr>", size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFBB><td>Log File or Directory<br></td><td><input type=text name=str_log_name size=50 value=\"",
			size - strlen(dest));
		strncat(dest, log_struct.name, size - strlen(dest));
		strncat(dest, "\"></td></tr>", size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFBB><td>Log Name Format:<br />(optional) ",size - strlen(dest));
//DMM
		strncat(dest,"<a href=\"javascript:void(0)\" onClick=\"myWindow = window.open('', 'tinyWindow', 'width=350,height=300'); "
					"myWindow.document.write('<html><body><p>A percent sign (%) is used the represent the date format YYMMDD. Wildcards are acceptable.</p>"
					"<p>e.g. log names like ISALOG_20060913_WEB_000.w3c would be represented as ISALOG_20%_WEB_*.w3c).</p>"
					"If this field is not defined, the first matching entry will be used (this is fine in most cases).</body></html>'); "
					"myWindow.document.close();"
					"\">Help</a>", size - strlen(dest));

		strncat(dest,"</td><td><input type=text name=str_log_format size=50 value=\"",
			size - strlen(dest));
		strncat(dest, log_struct.format, size - strlen(dest));
		strncat(dest, "\"></td></tr>", size - strlen(dest));

		strncat(dest, "</table><br>", size - strlen(dest));
		strncat(dest, "<input type=hidden name=lognumber value=", size - strlen(dest));
		strncat(dest, str_temp_log, size - strlen(dest));	// Log number goes here
		strncat(dest, "><input type=submit value=\"Change Configuration\">    ", size - strlen(dest));
		strncat(dest, "<input type=reset value=\"Reset Form\"></form>", size - strlen(dest));
	} else {

		dw_log_delete_error = Delete_Log(i_log_count);
		i_log_count++;
		dw_log_error = Read_Log_Registry(i_log_count,&log_struct);
		while (dw_log_error == 0)
		{
			dw_log_write_error = Write_Log_Registry(i_log_count-1,&log_struct);
			dw_log_delete_error = Delete_Log(i_log_count);
			i_log_count++;
			dw_log_error = Read_Log_Registry(i_log_count,&log_struct);
		}
		if (dw_log_delete_error == 0)
			strncat(dest,
				"<br><b>The log monitor has been removed<br><a href=\"log\">Return to Log display</a>.",
				size - strlen(dest));
		else
			strncat(dest,
				"<br>The log monitor was unable to be deleted.",
				size - strlen(dest));
	}

	return (0);
}

int Log_Result(char *source, char *dest, int size)
{
	// All strncpy or strncat functions in this routine have been designed avoid overflows
	Reg_Log log_struct, log_struct_read;
	int dw_log_error = 0;
	int i_log = 0, i_log_count = 0;
	char str_log_count[10];
	char *psource = source, Variable[100], Argument[100];

	strncpy(dest, "<form action=setlog><H2><CENTER>SNARE Log Configuration</H2>", size);

	while ((psource =
		GetNextArgument(psource, Variable, sizeof (Variable), Argument,
				sizeof (Argument))) != (char *) NULL) {

		if (strstr(Variable, "str_log_name") != NULL) {
			strncpy(log_struct.name, Argument, sizeof (log_struct.name));
		}

		if (strstr(Variable, "str_log_format") != NULL) {
			strncpy(log_struct.format, Argument, sizeof (log_struct.format));
		}
		
		if (strstr(Variable, "lognumber") != NULL) {
			strncpy(str_log_count, Argument, sizeof (str_log_count));
		}
		if (strstr(Variable, "str_log_type") != NULL) {
			strncpy(log_struct.type, Argument, sizeof (log_struct.type));
		}
	}
	
	if (!dw_log_error) {

		i_log = atoi(str_log_count);

		//-2 = "Add a new log monitor"
		if (i_log == -2) {
			dw_log_error = Read_Log_Registry(i_log_count,&log_struct_read);
			while (dw_log_error == 0)
			{
				i_log_count++;
				dw_log_error = Read_Log_Registry(i_log_count,&log_struct_read);
			}
			i_log = i_log_count;
		}

		dw_log_error = Write_Log_Registry(i_log,&log_struct);

		if (dw_log_error == 0)
			strncat(dest, "<br>The log monitor has been modified/added.", size - strlen(dest));
		else
			strncat(dest, "<br>The log monitor was unable to be modified/added.", size - strlen(dest));
	}

	return (0);
}

int Current_Events(char *source, char *dest, int size)
{
	DWORD dwWaitRes=0;
	EventCache *myMsg=NULL;
	snprintf(dest,size,"<HTML><BODY><H2><CENTER>Current Events</H2></CENTER><P><center><br />");
	dwWaitRes = WaitForSingleObject(hMutex,500);
	strncat(dest,"<table border=1 cellspacing=0 cellpadding=2 width=\"99%\" bgcolor=\"white\">\n" \
		"<tr bgcolor=\"#ffffcc\"><td width=10%>Date</td><td width=10%>System</td><td width=10%>Type</td><td>Strings</td></tr>\n",size);
	if(dwWaitRes == WAIT_OBJECT_0) {
		myMsg = MCHead;
		int i=0;
		for (myMsg = MCHead; myMsg && i < 20; myMsg = myMsg->next, i++) {
			if (myMsg->seenflag) {
				if (i%2) snprintf(dest,size,"%s<tr bgcolor=#FEFEFE>", dest);
				else snprintf(dest,size,"%s<tr bgcolor=#EEEEEE>", dest);
			} else {
				if (i%2) snprintf(dest,size,"%s<tr bgcolor=#DDFFDD>", dest);
				else snprintf(dest,size,"%s<tr bgcolor=#CCEECC>", dest);
			}
			snprintf(dest,size,"%s<td>%s</td>", dest, myMsg->SubmitTime);
			snprintf(dest,size,"%s<td>%s</td>", dest, myMsg->Hostname);
			snprintf(dest,size,"%s<td>%s</td>", dest, myMsg->type);
			snprintf(dest,size,"%s<td>%s</td></tr>\n", dest, myMsg->msg);
			myMsg->seenflag=1;
		}
	}
	ReleaseMutex(hMutex);
	strncat(dest,"</table>",size);
	snprintf(dest,size,"%s</CENTER></BODY></HTML>",dest);
	return(0);
}